//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : uMain.pas
//Description : Example program for detecting bit changes (e.g. ME-8100, ME-8200, ME-5100/DIO)
//              - Open ME-iDS main driver
//              - list all sub-devices with Bit change capability
//              - Close ME-iDS main driver when program is closed
//              - interrupt status flag "normal" or "extended" can be selected
//                extended status is supported by ME-8100, ME-8200
//              - bits must be set according to the status flag set (normal or extended)
//                and the description of the hardware manual of the selected device
//              - the meaning of the bits that changed in the groupbox "Bit change indicator"
//                must be determined according to the hardware manual of the selected device
//==============================================================================
//Bitchange:
// This program can only work when an external signal is provided at the pins of the selected sub-device
// two options are supported to get notified about a bitchange on a subdevice
// - Callback function (using  meIOIrqSetCallback and a user defined callback function)
//   When a callback function is used, the driver creates an extra thread to execute the callback function
// - Wait explicitly for an interrupt to arrive (using meIOIrqWait)
//   Timeout is used to abort the meIOIrqWait function,
//   Timeout = 0 means wait indefinitely

unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, jpeg, ExtCtrls, meIDS, ComCtrls, CheckLst, ImgList, uWaitThread;

function meIOIrqCB(iDevice: Integer; iSubDevice: Integer; iChannel: Integer; iIrqCount: Integer;
                   iValue: Integer; pvContext: Pointer; iErrorCode: Integer): Integer; stdcall;

const
  WM_IRQ_OCCURED = WM_USER + 1;
  WM_IRQ_WAIT_ERROR = WM_USER + 2;

type

  //irq callback function thread info record
  _ThreadInfoCb = record
    idxDevice: Integer;
    idxSubdevice: Integer;
    idxChannel: Integer;
    iIrqCount: Integer;
    iValue: Integer;
    pvContext: Pointer;
    iErrorCode: Integer;
  end;
  TThreadInfoCb = _ThreadInfoCb;
  PThreadInfoCb = ^TThreadInfoCb;


//-------------------------------------------------
  TBitChSubdevice = class(TCollectionItem)
  private
    FDeviceName: string;
    FIndexDevice: Integer;
    FIndexSubdevice: Integer;
    FCapabilities: Integer;
    FNumberOfChannels: Integer;
    FSingleConfigFlag: Integer;
  published
    property DeviceName: string read FDeviceName;
    property IndexDevice: Integer read FIndexDevice;
    property IndexSubdevice: Integer read FIndexSubdevice;
    property Capabilities: Integer read FCapabilities;
    property NumberOfChannels: Integer read FNumberOfChannels;
    property SingleConfigFlag: Integer read FSingleConfigFlag;
  end;

  TBitChSubdevices = class(TCollection)
  private
    function GetSubdevice(Index: Integer): TBitChSubdevice;
    procedure SetSingleConfigFlag(ABitChSubdevice: TBitChSubdevice; ANumOfChannels: Integer);
  public
    function AddSubdevice(ADeviceName: string; AIdxDevice: Integer; AIdxSubdevice: Integer; ACapabilities: Integer; ANumOfChannels: Integer): TBitChSubdevice;
    property Items[Index: Integer]: TBitChSubdevice read GetSubdevice;
    Constructor Create;
  end;
//--End--------------------------------------------

  TfrmMain = class(TForm)
    btnClearHwInfo: TButton;
    btnAbout: TButton;
    btnClose: TButton;
    memoInfo: TMemo;
    grpBox1: TGroupBox;
    lbl3: TLabel;
    coBoxSubdevs: TComboBox;
    lbl1: TLabel;
    pnlTop: TPanel;
    lbl2: TLabel;
    btnReset: TButton;
    grpBox2: TGroupBox;
    rbUseCallbackFunc: TRadioButton;
    rbUseIRQWaitFunc: TRadioButton;
    lbl4: TLabel;
    editTimeout: TEdit;
    btnStartIRQ: TButton;
    btnStopIRQ: TButton;
    ud1: TUpDown;
    lbl5: TLabel;
    btnSelectAll: TButton;
    btnSelectNone: TButton;
    btnSelectInvert: TButton;
    chBoxBit0: TCheckBox;
    lbl6: TLabel;
    grpBoxEdges: TGroupBox;
    chBoxBit1: TCheckBox;
    chBoxBit3: TCheckBox;
    chBoxBit2: TCheckBox;
    lbl7: TLabel;
    chBoxBit4: TCheckBox;
    lbl8: TLabel;
    chBoxBit5: TCheckBox;
    chBoxBit7: TCheckBox;
    chBoxBit6: TCheckBox;
    lbl9: TLabel;
    chBoxBit8: TCheckBox;
    lbl10: TLabel;
    chBoxBit9: TCheckBox;
    chBoxBit11: TCheckBox;
    chBoxBit10: TCheckBox;
    lbl11: TLabel;
    chBoxBit12: TCheckBox;
    lbl12: TLabel;
    chBoxBit13: TCheckBox;
    chBoxBit15: TCheckBox;
    chBoxBit14: TCheckBox;
    lbl13: TLabel;
    chBoxBit16: TCheckBox;
    lbl14: TLabel;
    chBoxBit17: TCheckBox;
    chBoxBit19: TCheckBox;
    chBoxBit18: TCheckBox;
    lbl15: TLabel;
    chBoxBit20: TCheckBox;
    lbl16: TLabel;
    chBoxBit21: TCheckBox;
    chBoxBit23: TCheckBox;
    chBoxBit22: TCheckBox;
    lbl17: TLabel;
    chBoxBit24: TCheckBox;
    lbl18: TLabel;
    chBoxBit25: TCheckBox;
    chBoxBit27: TCheckBox;
    chBoxBit26: TCheckBox;
    lbl19: TLabel;
    chBoxBit28: TCheckBox;
    lbl20: TLabel;
    chBoxBit29: TCheckBox;
    chBoxBit31: TCheckBox;
    chBoxBit30: TCheckBox;
    lbl21: TLabel;
    coBoxEdges: TComboBox;
    grpBox3: TGroupBox;
    shpBit0: TShape;
    shpBit1: TShape;
    shpBit2: TShape;
    shpBit3: TShape;
    shpBit4: TShape;
    shpBit5: TShape;
    shpBit6: TShape;
    shpBit7: TShape;
    shpBit8: TShape;
    shpBit9: TShape;
    shpBit10: TShape;
    shpBit11: TShape;
    shpBit12: TShape;
    shpBit13: TShape;
    shpBit14: TShape;
    shpBit15: TShape;
    shpBit16: TShape;
    shpBit17: TShape;
    shpBit18: TShape;
    shpBit19: TShape;
    shpBit20: TShape;
    shpBit21: TShape;
    shpBit22: TShape;
    shpBit23: TShape;
    shpBit24: TShape;
    shpBit25: TShape;
    shpBit26: TShape;
    shpBit27: TShape;
    shpBit28: TShape;
    shpBit29: TShape;
    shpBit30: TShape;
    shpBit31: TShape;
    lbl22: TLabel;
    lbl23: TLabel;
    lbl24: TLabel;
    lbl25: TLabel;
    lbl26: TLabel;
    lbl27: TLabel;
    lbl28: TLabel;
    lbl29: TLabel;
    lbl30: TLabel;
    lbl31: TLabel;
    lbl32: TLabel;
    lbl33: TLabel;
    lbl34: TLabel;
    lbl35: TLabel;
    lbl36: TLabel;
    lbl37: TLabel;
    lbl38: TLabel;
    lbl39: TLabel;
    coBoxStatusFlag: TComboBox;
    btnSelectLowerBits: TButton;
    btnSelectUpperBits: TButton;
    btnClearIndicator: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure btnClearHwInfoClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure coBoxSubdevsChange(Sender: TObject);
    procedure btnStartIRQClick(Sender: TObject);
    procedure btnStopIRQClick(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure btnSelectAllClick(Sender: TObject);
    procedure btnSelectNoneClick(Sender: TObject);
    procedure btnSelectInvertClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnSelectUpperBitsClick(Sender: TObject);
    procedure btnSelectLowerBitsClick(Sender: TObject);
    procedure btnClearIndicatorClick(Sender: TObject);
  private
    { Private-Deklarationen }
    openErrSuccess: Integer;
    bitChangeSubdevices: TBitChSubdevices;
    lineNumber: Integer;
    bitChBoxes: array[0..31] of TCheckBox;
    bitShapes: array[0..31] of TShape;
    waitThread: TWaitThread;
    procedure OnWMInterruptOccured(var Msg: TMessage); message WM_IRQ_OCCURED;
    procedure OnWMIrqWaitErrorOccured(var Msg: TMessage); message WM_IRQ_WAIT_ERROR;
    procedure OnTerminateThread(Sender: TObject);
    function MeVersionToString(version: Integer): string;
    procedure GetSubdevicesWithBitchange(numDevices: Integer);
    procedure GetSubdevicesByType(idxDevice: Integer; subdeviceType: Integer);
    function GetSubdeviceTypeName(subdeviceType: Integer): string;
    procedure FillSubdeviceCaps(idxSelectedSubdev: Integer);
    procedure DisplayValues(idxDevice: Integer; idxSubdevice: Integer; idxChannel: Integer; irqCount: Integer;
                   iValue: Integer; pvContext: Pointer; errorCode: Integer);
    function IntToBin(val: Cardinal; dispDigits: Integer): string;
    procedure EnableUI(isEnabled: Boolean);
    procedure DisplayErrorMessage(ErrMsg: string; meError: Integer);
    procedure RaiseErrorMessage(ErrMsg: string; meError: Integer);
    procedure AddBitchangeSubdevice(devName: string; idxDevice: Integer;
      idxSubdevice: Integer; subdeviceType: Integer);
    procedure OpenMEiDSDriver();
    procedure CloseMEiDSDriver();
    function GetEdgesMask(): Integer;
    procedure DisplayBitIndicators(bitValue: Integer);
    procedure ClearBitIndicators;
    procedure DisplaySubdevices();
    function GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
    procedure ConfigureSubdeviceForInput(bitChangeDevice: TBitChSubdevice);
    procedure DisplayVersionLibrary();
    procedure DisplayVersionMainDriver;
    procedure UncheckBitMask;
    procedure RemoveCallbackFunction(bitChangeDevice: TBitChSubdevice);
    procedure WaitForInterrupt(bitChangeDevice: TBitChSubdevice; timeout: Integer);
    procedure TerminateWaitThread();
    procedure FillStatusFlags;
    function GetBitChangeCaps(): Integer;
    procedure InitBitMaskCheckBoxArray;
    procedure InitBitIndicatorsArray;
    procedure DisplayEdgesAndBitMask(irqArg: Integer);

  public
    { Public-Deklarationen }
  end;


var
  frmMain: TfrmMain;
  threadInfoCb: TThreadInfoCb;

implementation

uses meError, uAbout;

{$R *.dfm}

const
  ME_IDS_NOTOPENED = -1;

  COLOUR_INACTIV = clWhite;
  COLOUR_CHANGED = clGreen;

//-------------------------------------------------
//Callback function, called each time an IRQ is executed when bit change is detected
function meIOIrqCB(iDevice: Integer; iSubDevice: Integer; iChannel: Integer; iIrqCount: Integer;
                   iValue: Integer; pvContext: Pointer; iErrorCode: Integer): Integer; stdcall;
begin
  if (TForm(pvContext) is TfrmMain) and (iErrorCode = ME_ERRNO_SUCCESS) then
  begin
    threadInfoCb.idxDevice := iDevice;
    threadInfoCb.idxSubdevice := iSubDevice;
    threadInfoCb.idxChannel := iChannel;
    threadInfoCb.iIrqCount := iIrqCount;
    threadInfoCb.iValue := iValue;
    threadInfoCb.pvContext := pvContext;
    threadInfoCb.iErrorCode := iErrorCode;

    PostMessage(TfrmMain(pvContext).Handle, WM_IRQ_OCCURED, 0, iDevice);
  end;
  //return the error code, so if an error occured the driver will stop calling the callback function
  Result := iErrorCode;
end;

//-------------------------------------------------
{TBitChDevice}
Constructor TBitChSubdevices.Create();
begin
  inherited Create(TBitChSubdevice);
end;

procedure TBitChSubdevices.SetSingleConfigFlag(ABitChSubdevice: TBitChSubdevice; ANumOfChannels: Integer);
begin
  case ANumOfChannels of
    1:
      begin
        ABitChSubdevice.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_BIT;
      end;
    8:
      begin
        ABitChSubdevice.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_BYTE;
      end;
    16:
      begin
        ABitChSubdevice.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_WORD;
      end;
    24:
      begin
        ABitChSubdevice.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_DWORD;
      end;
    32:
      begin
        ABitChSubdevice.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_DWORD;
      end;
  else
    ABitChSubdevice.FSingleConfigFlag := ME_VALUE_INVALID;
  end;
end;

function TBitChSubdevices.GetSubdevice(Index: Integer): TBitChSubdevice;
begin
  Result := TBitChSubdevice(inherited GetItem(Index));
end;

function TBitChSubdevices.AddSubdevice(ADeviceName: string; AIdxDevice: Integer; AIdxSubdevice: Integer; ACapabilities: Integer; ANumOfChannels: Integer): TBitChSubdevice;
begin
  Result := inherited Add as TBitChSubdevice;
  Result.FDeviceName := ADeviceName;
  Result.FIndexDevice := AIdxDevice;
  Result.FIndexSubdevice := AIdxSubdevice;
  Result.FCapabilities := ACapabilities;
  Result.FNumberOfChannels := ANumOfChannels;
  SetSingleConfigFlag(Result, ANumOfChannels);
end;

//----------------------------------------------
{TfrmMain}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  openErrSuccess := ME_IDS_NOTOPENED;
  lineNumber := 0;
  FillStatusFlags;

  bitChangeSubdevices := TBitChSubdevices.Create;
  InitBitMaskCheckBoxArray;
  InitBitIndicatorsArray;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  OpenMEiDSDriver();
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseMEiDSDriver();
  bitChangeSubdevices.Free;
end;


procedure TfrmMain.OpenMEiDSDriver();
var
  numDevices: Integer;
begin
  memoInfo.Lines.Append('Opening ME-iDS...');

  openErrSuccess := meOpen(ME_OPEN_NO_FLAGS);
  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    DisplayVersionLibrary();
    DisplayVersionMainDriver();
    memoInfo.Lines.Append('');

    meQueryNumberDevices(numDevices);
    memoInfo.Lines.Append('Number of devices found: ' + IntToStr(numDevices));

    GetSubdevicesWithBitchange(numDevices);
  end else
  begin
    DisplayErrorMessage('Error opening ME-iDS. Error: ', openErrSuccess);
  end;
end;

procedure TfrmMain.GetSubdevicesWithBitchange(numDevices: Integer);
var
  meError: Integer;
  idxDevice: Integer;
  plugged: Integer;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (numDevices > 0) then
  begin

    memoInfo.Lines.Add('Retrieving sub-devices of type ME_TYPE_DI (Digital In) and ME_TYPE_DIO (Digital In/Out) and bitchange capability...');
    for idxDevice := 0 to numDevices - 1 do
    begin
      meError := GetDevicePlugged(idxDevice, plugged);

      if (meError = ME_ERRNO_SUCCESS) and (plugged = ME_PLUGGED_IN) then
      begin
        GetSubdevicesByType(idxDevice, ME_TYPE_DI);
        GetSubdevicesByType(idxDevice, ME_TYPE_DIO);
      end else
      begin
        memoInfo.Lines.Append('Device ' + IntToStr(idxDevice) + ' not plugged in.');
      end;
    end;
  end;
  DisplaySubdevices();
end;

procedure TfrmMain.GetSubdevicesByType(idxDevice: Integer; subdeviceType: Integer);
var
  idxSubDevice: Integer;
  numSubDevices: Integer;
  meError: Integer;
  idxMatchedSubdevice: Integer;
  devName: array[0..ME_DEVICE_NAME_MAX_COUNT] of AnsiChar;
begin
  meError := meQueryNumberSubdevices(idxDevice, numSubDevices);
  if meError = ME_ERRNO_SUCCESS then
  begin
    meQueryNameDevice(idxDevice, devName, ME_DEVICE_NAME_MAX_COUNT);
    idxSubDevice := 0;
    while idxSubDevice < numSubDevices do
    begin
      meError := meQuerySubdeviceByType(idxDevice,
                                        idxSubDevice,
                                        subdeviceType,
                                        ME_SUBTYPE_ANY,
                                        idxMatchedSubdevice);

      if meError = ME_ERRNO_SUCCESS then
      begin
        AddBitchangeSubdevice(string(devName), idxDevice, idxMatchedSubdevice, subdeviceType);
        idxSubDevice := idxMatchedSubdevice + 1;
      end else
      begin
        Break;
      end;
    end;
  end;
end;

procedure TfrmMain.AddBitchangeSubdevice(devName: string; idxDevice: Integer; idxSubdevice: Integer; subdeviceType: Integer);
var
  meError: Integer;
  subdevCaps: Integer;
  numOfChannels: Integer;
  desiredCaps: Integer;
begin
  desiredCaps := GetBitChangeCaps();

  meError := meQuerySubdeviceCaps(idxDevice,
                                  idxSubdevice,
                                  subdevCaps);
  if (meError = ME_ERRNO_SUCCESS) and ((subdevCaps and desiredCaps) <> 0) then
  begin
    meQueryNumberChannels(idxDevice, idxSubdevice, numOfChannels);
    bitChangeSubdevices.AddSubdevice(string(devName), idxDevice, idxSubdevice, subdevCaps, numOfChannels);

    coBoxSubdevs.Items.Append(string(devName)
                                + ',  Device ' + IntToStr(idxDevice)
                                + ',  Subdevice ' + IntToStr(idxSubdevice)
                                + ',  Type: ' + GetSubdeviceTypeName(subdeviceType)
                                + ',  Channels: ' + IntToStr(numOfChannels));
  end;
end;

procedure TfrmMain.coBoxSubdevsChange(Sender: TObject);
begin
  FillSubdeviceCaps(coBoxSubdevs.ItemIndex);
end;

procedure TfrmMain.FillSubdeviceCaps(idxSelectedSubdev: Integer);
var
  bitChangeDevice: TBitChSubdevice;
  subdevCaps: Integer;
begin
  if idxSelectedSubdev >= 0 then
  begin
    bitChangeDevice := bitChangeSubdevices.Items[idxSelectedSubdev];
    subdevCaps := bitChangeDevice.Capabilities;
    coBoxEdges.Clear;
    if (subdevCaps and ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING) = ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING then
    begin
      coBoxEdges.AddItem('Rising', Pointer(ME_IRQ_EDGE_RISING));
    end;
    if (subdevCaps and ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING) = ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING then
    begin
      coBoxEdges.AddItem('Falling', Pointer(ME_IRQ_EDGE_FALLING));
    end;
    if (subdevCaps and ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) = ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY then
    begin
      coBoxEdges.AddItem('Any', Pointer(ME_IRQ_EDGE_ANY));
    end;
    coBoxEdges.ItemIndex := coBoxEdges.Items.Count -1;

    UncheckBitMask();
    ClearBitIndicators();
  end
end;

procedure TfrmMain.OnWMInterruptOccured(var msg: TMessage);
begin
  DisplayValues(threadInfoCb.idxDevice,
                threadInfoCb.idxSubdevice,
                threadInfoCb.idxChannel,
                threadInfoCb.iIrqCount,
                threadInfoCb.iValue,
                threadInfoCb.pvContext,
                threadInfoCb.iErrorCode);
end;

procedure TfrmMain.OnWMIrqWaitErrorOccured(var Msg: TMessage);
begin
  DisplayErrorMessage('meIOIrqWait Error: ', Integer(Msg.LParam));
end;

procedure TfrmMain.DisplayValues(idxDevice: Integer; idxSubdevice: Integer; idxChannel: Integer; irqCount: Integer;
                   iValue: Integer; pvContext: Pointer; errorCode: Integer);
begin
  inc(lineNumber);
  memoInfo.lines.add(Format('%.2d', [lineNumber])
                        + ' Bitchange on Device: ' + IntToStr(idxDevice)
                        + ',  Subdevice: ' + IntToStr(idxSubdevice)
                        + ',  Value hex: ' + IntToHex(iValue, 8)
                        + ',  bin: ' + IntToBin(iValue, 32)
                        + ',  dec: ' + IntToStr(iValue)
                        );
  DisplayBitIndicators(iValue);
end;

procedure TfrmMain.DisplayBitIndicators(bitValue: Integer);
var
  i: Integer;
  bitPos: Integer;
begin
  bitPos := 1;
  for i := 0 to 31 do
  begin
    if (bitValue and bitPos) = bitPos then
    begin
      bitShapes[i].Brush.Color := COLOUR_CHANGED;
    end else
    begin
      bitShapes[i].Brush.Color := COLOUR_INACTIV;
    end;
    bitPos := bitPos shl 1;
  end;
end;

procedure TfrmMain.EnableUI(isEnabled: Boolean);
begin
  btnStartIRQ.Enabled := isEnabled;
  btnStopIRQ.Enabled := not isEnabled;
  coBoxSubdevs.Enabled := isEnabled;
  btnSelectAll.Enabled := isEnabled;
  btnSelectNone.Enabled := isEnabled;
  btnSelectInvert.Enabled := isEnabled;
  btnSelectUpperBits.Enabled := isEnabled;
  btnSelectLowerBits.Enabled := isEnabled;
  coBoxEdges.Enabled := isEnabled;
  coBoxStatusFlag.Enabled := isEnabled;
  grpBoxEdges.Enabled := isEnabled;
end;

procedure TfrmMain.btnStartIRQClick(Sender: TObject);
var
  meError: Integer;
  bitChangeDevice: TBitChSubdevice;
  idxDevice: Integer;
  idxSubdevice: Integer;
  irqArg: Integer;
  irqEdge: Integer;
  irqStatusFlag: Integer;
  timeOut: Integer;
begin
  if (coBoxSubdevs.ItemIndex >= 0) then
  begin
    bitChangeDevice := bitChangeSubdevices.Items[coBoxSubdevs.ItemIndex];

    idxDevice := bitChangeDevice.IndexDevice;
    idxSubdevice := bitChangeDevice.IndexSubdevice;
    irqEdge := Integer(coBoxEdges.Items.Objects[coBoxEdges.ItemIndex]);
    irqStatusFlag := Integer(coBoxStatusFlag.Items.Objects[coBoxStatusFlag.ItemIndex]);
    irqArg := GetEdgesMask();

    ConfigureSubdeviceForInput(bitChangeDevice);
    EnableUI(False);
    DisplayEdgesAndBitMask(irqArg);

    if rbUseCallbackFunc.Checked then
    begin
      meIOIrqSetCallback(idxDevice, idxSubdevice, meIOIrqCB, Self, ME_IO_IRQ_SET_CALLBACK_NO_FLAGS);
      meError := meIOIrqStart(idxDevice, idxSubdevice, 0, ME_IRQ_SOURCE_DIO_MASK, irqEdge, irqArg, irqStatusFlag);
    end else
    begin
      RemoveCallbackFunction(bitChangeDevice);
      try
        timeOut := StrToInt(editTimeout.Text);
        meError := meIOIrqStart(idxDevice, idxSubdevice, 0, ME_IRQ_SOURCE_DIO_MASK, irqEdge, irqArg, irqStatusFlag);
        if meError = ME_ERRNO_SUCCESS then
        begin
          WaitForInterrupt(bitChangeDevice, timeOut);
        end;
      except
        on e: EConvertError do
        begin
          memoInfo.Lines.Add(e.ClassName + ' ' + e.Message);
          EnableUI(True);
          Exit;
        end;
      end;
    end;

    if meError <> ME_ERRNO_SUCCESS then
    begin
      EnableUI(True);
      DisplayErrorMessage('Error: ', meError);
    end;
  end;
end;

procedure TfrmMain.WaitForInterrupt(bitChangeDevice: TBitChSubdevice; timeout: Integer);
begin
  waitThread := TWaitThread.Create(bitChangeDevice.IndexDevice,
                                   bitChangeDevice.IndexSubdevice,
                                   Self,
                                   timeout);
  waitThread.OnTerminate := OnTerminateThread;
  waitThread.Start;
end;

procedure TfrmMain.OnTerminateThread(Sender: TObject);
begin
  EnableUI(True);
  waitThread := nil;
end;

procedure TfrmMain.ConfigureSubdeviceForInput(bitChangeDevice: TBitChSubdevice);
var
  meError: Integer;
begin
  meError := meIOSingleConfig(bitChangeDevice.IndexDevice,
                          bitChangeDevice.IndexSubdevice,
                          0,                                          
                          ME_SINGLE_CONFIG_DIO_INPUT,
                          ME_REF_NONE,
                          ME_TRIG_CHAN_NONE,                          
                          ME_TRIG_TYPE_NONE,
                          ME_TRIG_EDGE_NONE,
                          bitChangeDevice.SingleConfigFlag);
  if meError <> ME_ERRNO_SUCCESS then
  begin
    RaiseErrorMessage('meIOSingleConfig Error: ', meError)
  end;
end;

procedure TfrmMain.RemoveCallbackFunction(bitChangeDevice: TBitChSubdevice);
begin
  meIOIrqSetCallback(bitChangeDevice.IndexDevice,
                     bitChangeDevice.IndexSubdevice,
                     nil,
                     nil,
                     ME_IO_IRQ_SET_CALLBACK_NO_FLAGS);
end;

procedure TfrmMain.DisplayVersionLibrary();
var
  ver: Integer;
begin
  meQueryVersionLibrary(ver);
  memoInfo.Lines.Append('Library version: ' + MeVersionToString(ver));
end;

procedure TfrmMain.DisplayVersionMainDriver();
var
  ver: Integer;
begin
  meQueryVersionMainDriver(ver);
  memoInfo.Lines.Append('Main driver version: ' + MeVersionToString(ver));
end;

function TfrmMain.GetEdgesMask(): Integer;
var
  i: Integer;
  irqArg: Integer;
begin
  irqArg := 0;
  for i := 31 downto 0 do
  begin
    if bitChBoxes[i].Checked then
      irqArg := irqArg or (1 shl i);
  end;
  Result := irqArg;
  if irqArg = 0 then
  begin
    raise Exception.Create('Please check the bits in the Edges Bit Mask that will detect a bit change.');
  end;
end;

procedure TfrmMain.ClearBitIndicators;
var
  i: Integer;
begin
  for i := 0 to 31 do
  begin
    bitShapes[i].Brush.Color := COLOUR_INACTIV;
  end;
end;

procedure TfrmMain.DisplaySubdevices();
begin
  if coBoxSubdevs.Items.Count > 0 then
  begin
    memoInfo.Lines.Append('Number of sub-devices with bit change found: ' + IntToStr(coBoxSubdevs.Items.Count));
    memoInfo.Lines.AddStrings(coBoxSubdevs.Items);
    memoInfo.Lines.Add('');

    coBoxSubdevs.ItemIndex := 0;
    FillSubdeviceCaps(coBoxSubdevs.ItemIndex);
    btnReset.Enabled := True;
    EnableUI(True);
  end else
  begin
    memoInfo.Lines.Append('No sub-devices with bit change capability found.');
    CloseMEiDSDriver;
  end;
end;

function TfrmMain.GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
var
  busNo: Integer;
  vendorId: Integer;
  deviceId: Integer;
  funcNo: Integer;
  busType: Integer;
  serialNo: Integer;
  devNo: Integer;
begin
  Result := meQueryInfoDevice(idxDevice,
                    vendorId,
                    deviceId,
                    serialNo,
                    busType,
                    busNo,
                    devNo,
                    funcNo,
                    plugged);
end;

procedure TfrmMain.btnStopIRQClick(Sender: TObject);
var
  bitChangeDevice: TBitChSubdevice;
  idxDevice: Integer;
  idxSubdevice: Integer;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (coBoxSubdevs.ItemIndex >= 0) then
  begin
    bitChangeDevice := bitChangeSubdevices.Items[coBoxSubdevs.ItemIndex];
    idxDevice := bitChangeDevice.IndexDevice;
    idxSubdevice := bitChangeDevice.IndexSubdevice;
    meIOIrqStop(idxDevice, idxSubdevice, 0, ME_IO_IRQ_STOP_NO_FLAGS);
    TerminateWaitThread();
    EnableUI(True);
    memoInfo.Lines.Add('Stop IRQ on Device ' + IntToStr(idxDevice) + ', Subdevice ' + IntToStr(idxSubdevice));
  end;
  ClearBitIndicators;
end;

procedure TfrmMain.TerminateWaitThread();
begin
  if Assigned(waitThread) then
  begin
    waitThread.Terminate;
  end;
end;

procedure TfrmMain.btnResetClick(Sender: TObject);
var
  meError: Integer;
  bitChangeDevice: TBitChSubdevice;
  idxDevice: Integer;
  idxSubdevice: Integer;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (coBoxSubdevs.ItemIndex >= 0) then
  begin
    bitChangeDevice := bitChangeSubdevices.Items[coBoxSubdevs.ItemIndex];
    idxDevice := bitChangeDevice.IndexDevice;
    idxSubdevice := bitChangeDevice.IndexSubdevice;
    meError := meIOResetSubdevice(idxDevice,
                                  idxSubdevice,
                                  ME_IO_RESET_SUBDEVICE_NO_FLAGS);
    if meError = ME_ERRNO_SUCCESS then
    begin
      memoInfo.Lines.Add('Reset Device ' + IntToStr(idxDevice) + ', Subdevice ' + IntToStr(idxSubDevice));
      EnableUI(True);
    end else
    begin
      DisplayErrorMessage('meIOResetSubdevice - Error: ', meError);
    end;
    ClearBitIndicators;
  end;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.CloseMEiDSDriver();
begin
  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    meClose(ME_CLOSE_NO_FLAGS);
    openErrSuccess := ME_IDS_NOTOPENED;
    coBoxSubdevs.Clear;
    bitChangeSubdevices.Clear;
    EnableUI(False);
    btnReset.Enabled := False;
  end;
end;

procedure TfrmMain.DisplayErrorMessage(ErrMsg: string; meError: Integer);
var
  errorMsg: array[0..ME_ERROR_MSG_MAX_COUNT] of AnsiChar;
begin
  meErrorGetMessage(meError, errorMsg, ME_ERROR_MSG_MAX_COUNT);
  memoInfo.lines.add(ErrMsg + IntToStr(meError));
  memoInfo.Lines.Add('ErrorMessage: ' + errorMsg);
end;

procedure TfrmMain.RaiseErrorMessage(ErrMsg: string; meError: Integer);
var
  errorMsg: array[0..ME_ERROR_MSG_MAX_COUNT] of AnsiChar;
begin
  meErrorGetMessage(meError, errorMsg, ME_ERROR_MSG_MAX_COUNT);
  raise Exception.Create(ErrMsg + ' ' + string(errorMsg));
end;

procedure TfrmMain.FillStatusFlags;
begin
  coBoxStatusFlag.AddItem('Standard', TObject(ME_IO_IRQ_START_NO_FLAGS));
  coBoxStatusFlag.AddItem('Extended', TObject(ME_IO_IRQ_START_EXTENDED_STATUS));
  coBoxStatusFlag.ItemIndex := 0;
end;

procedure TfrmMain.DisplayEdgesAndBitMask(irqArg: Integer);
begin
  memoInfo.Lines.Add('StartIRQ - Flag: ' + coBoxStatusFlag.Text
                   + ' Edges: ' + coBoxEdges.Text
                   + ', BitMask: ' + IntToBin(irqArg, 32));
end;

procedure TfrmMain.btnClearHwInfoClick(Sender: TObject);
begin
  memoInfo.Clear;
  lineNumber := 0;
end;

procedure TfrmMain.btnSelectAllClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to 31 do
  begin
    bitChBoxes[i].Checked := True;
  end;
end;

procedure TfrmMain.btnSelectNoneClick(Sender: TObject);
begin
  UncheckBitMask;
end;

procedure TfrmMain.UncheckBitMask;
var
  i: Integer;
begin
  for i := 0 to 31 do
  begin
    bitChBoxes[i].Checked := False;
  end;
end;

procedure TfrmMain.btnSelectInvertClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to 31 do
  begin
    bitChBoxes[i].Checked := not bitChBoxes[i].Checked;
  end;
end;

procedure TfrmMain.btnSelectLowerBitsClick(Sender: TObject);
var
  i: Integer;
  bitChangeDevice: TBitChSubdevice;
  numBits: Integer;
begin
  if (coBoxSubdevs.ItemIndex >= 0) then
  begin
    bitChangeDevice := bitChangeSubdevices.Items[coBoxSubdevs.ItemIndex];
    numBits := bitChangeDevice.NumberOfChannels;
    if numBits > 16 then
    begin
      numBits := 16;
    end;
    for i := 0 to numBits -1 do
    begin
      bitChBoxes[i].Checked := True;
    end;
  end;
end;

procedure TfrmMain.btnSelectUpperBitsClick(Sender: TObject);
var
  i: Integer;
  bitChangeDevice: TBitChSubdevice;
  numBits: Integer;
begin
  if (coBoxSubdevs.ItemIndex >= 0) then
  begin
    bitChangeDevice := bitChangeSubdevices.Items[coBoxSubdevs.ItemIndex];
    numBits := bitChangeDevice.NumberOfChannels;
    if numBits > 16 then
    begin
      numBits := 16;
    end;
    for i := 16 to 16 + numBits -1 do
    begin
      bitChBoxes[i].Checked := True;
    end;
  end;
end;

procedure TfrmMain.btnClearIndicatorClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to 31 do
  begin
    bitShapes[i].Brush.Color := COLOUR_INACTIV;
  end;
end;

function TfrmMain.GetBitChangeCaps(): Integer;
begin
  Result := ME_CAPS_DIO_BIT_PATTERN_IRQ
         or ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING
         or ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING
         or ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY;
end;

procedure TfrmMain.InitBitMaskCheckBoxArray;
begin
  bitChBoxes[0] := chBoxBit0;
  bitChBoxes[1] := chBoxBit1;
  bitChBoxes[2] := chBoxBit2;
  bitChBoxes[3] := chBoxBit3;
  bitChBoxes[4] := chBoxBit4;
  bitChBoxes[5] := chBoxBit5;
  bitChBoxes[6] := chBoxBit6;
  bitChBoxes[7] := chBoxBit7;
  bitChBoxes[8] := chBoxBit8;
  bitChBoxes[9] := chBoxBit9;
  bitChBoxes[10] := chBoxBit10;
  bitChBoxes[11] := chBoxBit11;
  bitChBoxes[12] := chBoxBit12;
  bitChBoxes[13] := chBoxBit13;
  bitChBoxes[14] := chBoxBit14;
  bitChBoxes[15] := chBoxBit15;
  bitChBoxes[16] := chBoxBit16;
  bitChBoxes[17] := chBoxBit17;
  bitChBoxes[18] := chBoxBit18;
  bitChBoxes[19] := chBoxBit19;
  bitChBoxes[20] := chBoxBit20;
  bitChBoxes[21] := chBoxBit21;
  bitChBoxes[22] := chBoxBit22;
  bitChBoxes[23] := chBoxBit23;
  bitChBoxes[24] := chBoxBit24;
  bitChBoxes[25] := chBoxBit25;
  bitChBoxes[26] := chBoxBit26;
  bitChBoxes[27] := chBoxBit27;
  bitChBoxes[28] := chBoxBit28;
  bitChBoxes[29] := chBoxBit29;
  bitChBoxes[30] := chBoxBit30;
  bitChBoxes[31] := chBoxBit31;
end;

procedure TfrmMain.InitBitIndicatorsArray;
begin
  bitShapes[0] := shpBit0;
  bitShapes[1] := shpBit1;
  bitShapes[2] := shpBit2;
  bitShapes[3] := shpBit3;
  bitShapes[4] := shpBit4;
  bitShapes[5] := shpBit5;
  bitShapes[6] := shpBit6;
  bitShapes[7] := shpBit7;
  bitShapes[8] := shpBit8;
  bitShapes[9] := shpBit9;
  bitShapes[10] := shpBit10;
  bitShapes[11] := shpBit11;
  bitShapes[12] := shpBit12;
  bitShapes[13] := shpBit13;
  bitShapes[14] := shpBit14;
  bitShapes[15] := shpBit15;
  bitShapes[16] := shpBit16;
  bitShapes[17] := shpBit17;
  bitShapes[18] := shpBit18;
  bitShapes[19] := shpBit19;
  bitShapes[20] := shpBit20;
  bitShapes[21] := shpBit21;
  bitShapes[22] := shpBit22;
  bitShapes[23] := shpBit23;
  bitShapes[24] := shpBit24;
  bitShapes[25] := shpBit25;
  bitShapes[26] := shpBit26;
  bitShapes[27] := shpBit27;
  bitShapes[28] := shpBit28;
  bitShapes[29] := shpBit29;
  bitShapes[30] := shpBit30;
  bitShapes[31] := shpBit31;
end;

function TfrmMain.IntToBin(val: Cardinal; dispDigits: Integer): string;
var
  i: Integer;
  b: Integer;
  tmpStr: string;
const
  binStr: array[0..15] of string = ('0000', '0001', '0010', '0011',
                                    '0100', '0101', '0110', '0111',
                                    '1000', '1001', '1010', '1011',
                                    '1100', '1101', '1110', '1111');
begin
  tmpStr := '';
  if dispDigits in [1, 8, 16, 24, 32] then
  begin
    if dispDigits = 1 then
    begin
      tmpStr := IntToStr(val and 1);
    end else
    begin
      for i := (dispDigits shr 2) -1 downto 0 do
      begin
        b := (val shr (i shl 2)) and $0F;
        tmpStr := tmpStr + binStr[b];
      end;
    end;
  end;
  Result := tmpStr;
end;

function TfrmMain.GetSubdeviceTypeName(subdeviceType: Integer): string;
begin
  case subdeviceType of
    ME_TYPE_INVALID: Result := 'ME_TYPE_INVALID';
    ME_TYPE_AI: Result := 'ME_TYPE_AI';
    ME_TYPE_AO: Result := 'ME_TYPE_AO';
    ME_TYPE_DIO: Result := 'ME_TYPE_DIO';
    ME_TYPE_DO: Result := 'ME_TYPE_DO';
    ME_TYPE_DI: Result := 'ME_TYPE_DI';
    ME_TYPE_CTR: Result := 'ME_TYPE_CTR';
    ME_TYPE_EXT_IRQ: Result := 'ME_TYPE_EXT_IRQ';
    ME_TYPE_FIO: Result := 'ME_TYPE_FIO';
    ME_TYPE_FO: Result := 'ME_TYPE_FO';
    ME_TYPE_FI: Result := 'ME_TYPE_FI';
    ME_TYPE_FPGA: Result := 'ME_TYPE_FPGA';
  else
    Result := '$' + IntToHex(subdeviceType, 8);
  end;
end;

function TfrmMain.MeVersionToString(version: Integer): string;
begin
  Result := Format('%d.%d.%d.%d', [version shr 24, $FF and (version shr 16), $FF and (version shr 8), $FF and version]);
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  frmAbout := TfrmAbout.Create(Self);
  try
    frmAbout.PopupParent := Self;
    frmAbout.ShowModal;
  finally
    frmAbout.Free;
  end;
end;

end.
