//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : meIDSToolsDLL.cs
//              contains a collection of routines to carry out common tasks using the ME-iDS system
//              The library is located in the folder:
//              %ProgramFiles%\Meilhaus Electronic\ME-iDS\[me-ids-version]\System\ME-iDS (Driver)
//              replace [me-ids-version] with the installed version
//              of the ME-iDS, e.g.: 02.00.03.000
//              See ME-iDS help file for more information
//Note        : import library for 32bit: meIDSTools.dll
//              import library for 64bit: meIDSTools64.dll
//              
//==============================================================================
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace meIDSTools_CS
{
    /// <summary>
    /// Defines all constants and methods to use the meIDSTools library
    /// </summary>
    public sealed class meIDSTools
    {
        #region Constants

        //==============================================================================
        //  Error codes
        //==============================================================================
        
        public const int MEIDS_TOOLS_ERRNO_SUCCESS                        = 0;
        public const int MEIDS_TOOLS_ERRNO_WAVEGEN_SIGNAL_OUT_OF_RANGE    = 10001;
        public const int MEIDS_TOOLS_ERRNO_WAVEGEN_FREQUENCY_OUT_OF_RANGE = 10002;
        
        //==============================================================================
        // Signal forms for the function meIDSToolsWaveGen - Parameter iShape
        //==============================================================================
        
        public const int MEIDS_TOOLS_WAVEGEN_SHAPE_RECTANGLE              = 0x10010001;
        public const int MEIDS_TOOLS_WAVEGEN_SHAPE_TRIANGLE               = 0x10010002;
        public const int MEIDS_TOOLS_WAVEGEN_SHAPE_SINUS                  = 0x10010003;
        public const int MEIDS_TOOLS_WAVEGEN_SHAPE_COSINUS                = 0x10010004;
        public const int MEIDS_TOOLS_WAVEGEN_SHAPE_POS_RAMP               = 0x10010005;
        public const int MEIDS_TOOLS_WAVEGEN_SHAPE_NEG_RAMP               = 0x10010006;

        #endregion

    #region meIDSToolsFunctions        
    /// <summary>
    /// Description of the library functions
    /// </summary>

        #region meIDSToolsWaveGen

        /// <summary>
        /// Creates the waveform using a number of data points less than the size of the internal FIFO on the sub-device.
        /// </summary>
        /// <param name="device">
        /// Index of the AO streaming device which will be used to output the waveform.
        /// </param>
        /// <param name="subdevice">
        /// Index of the sub-device to be accessed.
        /// </param>
        /// <param name="unit">
        /// Type of signal which will be output.
        /// Use one of the constants ME_UNIT_VOLT, ME_UNIT_AMPERE, ME_UNIT_HZ from meIDS.cs
        /// </param>
        /// <param name="shape">
        /// Shape of the waveform to be output
        /// </param>
        /// <param name="amplitude">
        /// The amplitude in the units of the parameter 'unit' of the signal produced
        /// </param>
        /// <param name="offset">
        /// The offset in the units of the parameter 'unit' of the signal produced
        /// </param>
        /// <param name="frequency">
        /// The frequency in Hz of the signal produced.
        /// </param>
        /// <returns>
        /// Returns one of the standard ME-iDS error codes or one of the error codes defined in this module
        /// </returns>
        /// <remarks>
        /// The routine uses as much of the internal FIFO as possible in order to produce a signal as smooth as possible.
        /// </remarks>
        [DllImport("meIDSTools.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int meIDSToolsWaveGen(int device, int subdevice, int unit, int shape, double amplitude, double offset, out double frequency);

        #endregion

    #endregion 

    }
}

