// TestConsole_meIDSSystemProperties.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <windows.h>

#include <stdio.h>
#include <conio.h>

#include <memory.h>

extern "C"
{
#include "..\\medriver.h"
}

#ifdef WIN64

#pragma comment (lib, "..\\meIDSmain64.lib")

#else

#pragma comment (lib, "..\\meIDSmain.lib")

#endif

int iIndent;

char szIndent[64];

int PrintContainerProperties(char* szContainerPath);

int _tmain(int argc, _TCHAR* argv[])
{
	// Increase the size of the console buffer
	
	HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

	COORD coord;

	coord.X = 160;
	
	coord.Y = 2000;
	
	SetConsoleScreenBufferSize(h_console, coord);

	printf("Press any key to continue\n");

	getch();

	// Enable the default error procedure

	meErrorSetDefaultProc(ME_SWITCH_ENABLE);

	// Open the driver system

	int i_me_error = meOpen(0);

	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("****    meOpen - Error: %d    ****\n\n", i_me_error);
		
		printf("Press any key to terminate\n\n");
		
		_getch();

		return(-1);
	}

	iIndent = 0;

	szIndent[0] = 0;

	// Use the routine PrintContainerProperties below to print the properties of the root container "\\".

	// As this routine is recursive (it is called internally recursvely for each container found in the 
	// container it is processing) it will thus print all the proerties found in the system.

	i_me_error = PrintContainerProperties("\\");

	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("PrintContainerProperties, <%s> - Error: %d\n", "\\", i_me_error);
	}

	meClose(0);
	
	printf(	"Press any key to terminate\n\n");

	getch();
			
	return 0;
}

int PrintContainerProperties(char* szContainerPath)
{
	char sz_buffer[1024];

	strcpy_s(&sz_buffer[0], 1024, szContainerPath);

	strcat_s(&sz_buffer[0], 1024, "\\NumberOfElements");

	int i_number_of_elements;
	
	int i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_number_of_elements);

	if(i_me_error != ME_ERRNO_SUCCESS)
	{
		printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
	}

	for(int index_element = 0; index_element < i_number_of_elements; index_element++)
	{
		sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\PropertyName", szContainerPath, index_element);

		char sz_name[64];

		i_me_error = mePropertyGetStringA(&sz_buffer[0], &sz_name[0], 64);

		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("mePropertyGetStringA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
		}

		sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\PropertyType", szContainerPath, index_element);

		int i_property_type;

		i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_property_type);

		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
		}

		sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\PropertyDescription", szContainerPath, index_element);

		char sz_property_description[1024];

		i_me_error = mePropertyGetStringA( &sz_buffer[0], &sz_property_description[0], 1024 );

		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("mePropertyGetStringA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
		}

		if(i_property_type == ME_PROPERTY_TYPE_CONTAINER)
		{
			printf("%s%s - Beschreibung: %s\n", &szIndent[0], &sz_name[0], &sz_property_description[0]); 

			sprintf_s(&sz_buffer[0], 1024, "%s\\%s", szContainerPath, &sz_name[0]);

			iIndent+= 4;

			memset(&szIndent[0], ' ', iIndent);

			szIndent[iIndent] = 0;

			i_me_error = PrintContainerProperties( &sz_buffer[0] );

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				printf("PrintContainerProperties, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
			}

			iIndent-= 4;

			szIndent[iIndent] = 0;
		}
		else
		{
			sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\PropertyAccess", szContainerPath, index_element);

			int i_access_type;

			i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_access_type);

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
			}

			sprintf_s(&sz_buffer[0], 1024, "%s\\%d", szContainerPath, index_element);

			switch(i_property_type)
			{
			case ME_PROPERTY_TYPE_BOOL:

				int b_value;

				i_me_error = mePropertyGetIntA(&sz_buffer[0], &b_value);

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
				}

				printf("%s%s    %s    %s - Beschreibung: %s\n", &szIndent[0], &sz_name[0], ( (b_value == 0) ? "False" : "True" ), ( (i_access_type == ME_PROPERTY_ACCESS_READ_ONLY) ? "Read only" : "Read/Write" ), &sz_property_description[0] ); 

				if(i_access_type == ME_PROPERTY_ACCESS_READ_WRITE)
				{
					i_me_error = mePropertySetIntA(&sz_buffer[0], b_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertySetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}
				}

				break;

			case ME_PROPERTY_TYPE_INT:

				int i_value;

				i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_value);

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
				}

				if(i_access_type == ME_PROPERTY_ACCESS_READ_WRITE)
				{
					i_me_error = mePropertySetIntA(&sz_buffer[0], i_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertySetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}
				}

				if(i_access_type == ME_PROPERTY_ACCESS_READ_ONLY)
				{
					printf("%s%s    %d    Read only - Beschreibung: %s\n", &szIndent[0], &sz_name[0], i_value, &sz_property_description[0]); 
				}
				else
				{
					sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\MinValue", szContainerPath, index_element);

					int i_min_value;

					i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_min_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}

					sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\MaxValue", szContainerPath, index_element);

					int i_max_value;

					i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_max_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}

					printf("%s%s    %d    (Range %d - %d) Read/Write - Beschreibung: %s\n", &szIndent[0], &sz_name[0], i_value, i_min_value, i_max_value, &sz_property_description[0]); 
				}

				break;

			case ME_PROPERTY_TYPE_DOUBLE:

				double d_value;

				i_me_error = mePropertyGetDoubleA(&sz_buffer[0], &d_value);

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("mePropertyGetDoubleA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
				}

				if(i_access_type == ME_PROPERTY_ACCESS_READ_WRITE)
				{
					i_me_error = mePropertySetDoubleA(&sz_buffer[0], d_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertySetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}
				}

				if(i_access_type == ME_PROPERTY_ACCESS_READ_ONLY)
				{
					printf("%s%s    %lg    Read only - Beschreibung: %s\n", &szIndent[0], &sz_name[0], d_value, &sz_property_description[0]); 
				}
				else
				{
					sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\MinValue", szContainerPath, index_element);

					double d_min_value;

					i_me_error = mePropertyGetDoubleA(&sz_buffer[0], &d_min_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertyGetDoubleA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}

					sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\MaxValue", szContainerPath, index_element);

					double d_max_value;

					i_me_error = mePropertyGetDoubleA(&sz_buffer[0], &d_max_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertyGetDoubleA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}

					printf("%s%s    %lg    (Range %lg - %lg) Read/Write - Beschreibung: %s\n", &szIndent[0], &sz_name[0], d_value, d_min_value, d_max_value, &sz_property_description[0]); 
				}

				break;

			case ME_PROPERTY_TYPE_STRING:

				char sz_value[512];

				i_me_error = mePropertyGetStringA(&sz_buffer[0], &sz_value[0], 512);

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("mePropertyGetStringA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
				}

				printf("%s%s    %s    %s - Beschreibung: %s\n", &szIndent[0], &sz_name[0], &sz_value[0], ( (i_access_type == ME_PROPERTY_ACCESS_READ_ONLY) ? "Read only" : "Read/Write" ), &sz_property_description[0] ); 

				break;

			case ME_PROPERTY_TYPE_DEFINE:

				i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_value);

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
				}

				if(i_access_type == ME_PROPERTY_ACCESS_READ_WRITE)
				{
					i_me_error = mePropertySetIntA(&sz_buffer[0], i_value);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertySetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}
				}

				i_me_error = mePropertyGetStringA(&sz_buffer[0], &sz_value[0], 512);

				if(i_me_error != ME_ERRNO_SUCCESS)
				{
					printf("mePropertyGetStringA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
				}

				if(i_access_type == ME_PROPERTY_ACCESS_READ_ONLY)
				{
					printf("%s%s    %s (0x%08X)    Read only - Beschreibung: %s\n", &szIndent[0], &sz_name[0], &sz_value[0], i_value, &sz_property_description[0]); 
				}
				else
				{
					printf("%s%s    %s (0x%08X)    ( Choices: ", &szIndent[0], &sz_name[0], &sz_value[0], i_value); 
			
					sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\NumberOfSelections", szContainerPath, index_element);

					int i_number_of_selections;

					i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_number_of_selections);

					if(i_me_error != ME_ERRNO_SUCCESS)
					{
						printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
					}

					for(int i_selection = 0; i_selection < i_number_of_selections; i_selection++)
					{
						sprintf_s(&sz_buffer[0], 1024, "%s\\%d\\Selection%d", szContainerPath, index_element, i_selection);

						i_me_error = mePropertyGetIntA(&sz_buffer[0], &i_value);

						if(i_me_error != ME_ERRNO_SUCCESS)
						{
							printf("mePropertyGetIntA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
						}

						i_me_error = mePropertyGetStringA(&sz_buffer[0], &sz_value[0], 512);

						if(i_me_error != ME_ERRNO_SUCCESS)
						{
							printf("mePropertyGetStringA, <%s> - Error: %d\n", &sz_buffer[0], i_me_error);
						}

						printf("%s (0x%08X)", &sz_value[0], i_value);

						if(i_selection < i_number_of_selections - 1)
						{
							printf(", ");
						}
					}

					printf(" ) Read/Write - Beschreibung: %s\n", &sz_property_description[0]);
				}

				break;
			}
		}
	}

	return 0;
}

