// Con_meUtilityPWM.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <stdio.h>
#include <conio.h>

#include <memory.h>

extern "C"
{
#include "..\\medriver.h"
}

#include "..\\TestConsoleGeneric.h"

#ifdef WIN64

#pragma comment (lib, "..\\meIDSmain64.lib")

#else

#pragma comment (lib, "..\\meIDSmain.lib")

#endif

int _tmain(int argc, _TCHAR* argv[])
{
	// Call generic console function in parent directory to 
	// open system and list the available devices and subdevices.

	int i_number_of_devices = TestConsoleMEIDSProlog();

	if(i_number_of_devices <= 0)
	{
		return(-1);
	}

	// ---------------------------------------------------------------------
	
	// Count the devices in the system which are candidates for PWM.
    // These must have at least three counter sub-devices.

	int i_number_of_pwm_devices = 0;

	int i_me_error;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

        int i_number_ctr_8254_subdevices = 0;
        
        int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_CTR,					// Type of sub-device to search for
												ME_SUBTYPE_CTR_8254,			// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			if(++i_number_ctr_8254_subdevices >= 3)
            {
                ++i_number_of_pwm_devices;

                break;
            }

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}

	if(i_number_of_pwm_devices == 0)
	{
		printf("****    No device for PWM found in system    ****\n\n");
		
		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	// Make a list of the devices which have at least three counter sub-devices, 
	// and note the indexes of the first three such sub-devices

	struct SPWMDevice
	{
		int	m_iDeviceIndex;

		int m_iNumberOfCounterSubdevices;

		int m_arrCounterSubdevices[3];
	};

	SPWMDevice* arrPWMDevices = new SPWMDevice[i_number_of_pwm_devices];

	i_number_of_pwm_devices = 0;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

        arrPWMDevices[i_number_of_pwm_devices].m_iDeviceIndex = index_device;

        arrPWMDevices[i_number_of_pwm_devices].m_iNumberOfCounterSubdevices = 0;
        
        int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_CTR,					// Type of sub-device to search for
												ME_SUBTYPE_CTR_8254,			// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			arrPWMDevices[i_number_of_pwm_devices].m_arrCounterSubdevices[ arrPWMDevices[i_number_of_pwm_devices].m_iNumberOfCounterSubdevices ] = index_subdevice;

            if(++arrPWMDevices[i_number_of_pwm_devices].m_iNumberOfCounterSubdevices >= 3)
			{
				++i_number_of_pwm_devices;

				break;
			}

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}

	printf("%d devices found for PWM:\n\n", i_number_of_pwm_devices);

	for(int index_device = 0; index_device < i_number_of_pwm_devices; index_device++)
	{
		printf(	"%2d.   On Device: %2d, sub-devices %2d, %2d and %2d are 8254 counter sub-devices\n",
				index_device + 1,
				arrPWMDevices[index_device].m_iDeviceIndex,
				arrPWMDevices[index_device].m_arrCounterSubdevices[0],
				arrPWMDevices[index_device].m_arrCounterSubdevices[1],
				arrPWMDevices[index_device].m_arrCounterSubdevices[2]		);
	}


	printf("\n\nPress any key to continue\n\n");

	getch();

	char buffer[512];
	
	while(true)
	{
		int i_list_index_device;

		bool b_terminate = false;

		while(true)
		{
			printf("Choose a device and sub-devices for PWM:\n\n");

			for(int index_device = 0; index_device < i_number_of_pwm_devices; index_device++)
			{
				printf(	"%2d.   Device: %2d, sub-devices %2d, %2d and %2d\n",
						index_device + 1,
						arrPWMDevices[index_device].m_iDeviceIndex,
						arrPWMDevices[index_device].m_arrCounterSubdevices[0],
						arrPWMDevices[index_device].m_arrCounterSubdevices[1],
						arrPWMDevices[index_device].m_arrCounterSubdevices[2]		);
			}

			printf("\nYour choice (1 - %2d) (RETURN to terminate): ", i_number_of_pwm_devices);  

			gets(&buffer[0]);

			printf("\n\n");
			
			if(buffer[0] == 0)
			{
				b_terminate = true;
				
				break;
			}
			
			if( (sscanf(&buffer[0], "%d", &i_list_index_device) != 1 ) )
			{
				continue;
			}

			if( (i_list_index_device < 1)||(i_list_index_device > i_number_of_pwm_devices) )
			{
				continue;
			}

			--i_list_index_device;

			break;
		}

		if(b_terminate)
		{
			break;
		}

		int i_ref;

		char* sz_ref;

		int i_iref_choice;
			
		while(true)
		{
			printf("Choose a value for 'iRef':\n\n");

			printf(	"1.   ME_REF_CTR_EXTERNAL\n"
					"2.   ME_REF_CTR_INTERNAL_1MHZ\n"
					"3.   ME_REF_CTR_INTERNAL_10MHZ\n"
					"4.   ME_REF_CTR_PREVIOUS\n"		); 

			printf("\nYour choice (1 - 4) (RETURN to terminate): ");  

			gets(&buffer[0]);

			printf("\n\n");
			
			if(buffer[0] == 0)
			{
				b_terminate = true;
				
				break;
			}
			
			if( (sscanf(&buffer[0], "%d", &i_iref_choice) != 1 ) )
			{
				continue;
			}

			if( (i_iref_choice < 1)||(i_iref_choice > 4) )
			{
				continue;
			}

			break;
		}

		if(b_terminate)
		{
			break;
		}

		switch(i_iref_choice)
		{
		case 1:

			i_ref = ME_REF_CTR_EXTERNAL;

			sz_ref = "ME_REF_CTR_EXTERNAL";

			break;

		case 2:

			i_ref = ME_REF_CTR_INTERNAL_1MHZ;

			sz_ref = "ME_REF_CTR_INTERNAL_1MHZ";

			break;

		case 3:

			i_ref = ME_REF_CTR_INTERNAL_10MHZ;

			sz_ref = "ME_REF_CTR_INTERNAL_10MHZ";

			break;

		case 4:

			i_ref = ME_REF_CTR_PREVIOUS;

			sz_ref = "ME_REF_CTR_PREVIOUS";

			break;
		}

		int i_prescaler;

		while(true)
		{
			printf("Please enter a pre-scaler between 1 and 65534 (RETURN to terminate): ");

			gets(&buffer[0]);

			printf("\n\n");

			if(buffer[0] == 0)
			{
				b_terminate = true;
				
				break;
			}
			
			if( (sscanf(&buffer[0], "%d", &i_prescaler) != 1 ) )
			{
				continue;
			}

			if( (i_prescaler < 1)||(i_prescaler > 65534) )
			{
				continue;
			}

			break;
		}

		if(b_terminate)
		{
			break;
		}

		int i_duty_cycle;

		while(true)
		{
			printf("Please enter a duty cycle between 1%% and 99%% (RETURN to terminate): ");

			gets(&buffer[0]);

			printf("\n\n");
			
			if(buffer[0] == 0)
			{
				b_terminate = true;
				
				break;
			}
			
			if( (sscanf(&buffer[0], "%d", &i_duty_cycle) != 1 ) )
			{
				continue;
			}

			if( (i_duty_cycle < 1)||(i_duty_cycle > 99) )
			{
				continue;
			}

			break;
		}

		if(b_terminate)
		{
			break;
		}

		int i_flag;

		char* sz_flag;

		int i_flag_choice;

		while(true)
		{
			printf("Choose a value for 'iREF':\n\n");

			printf(	"1.   ME_PWM_START_NO_FLAGS\n"
					"2.   ME_PWM_START_CONNECT_INTERNAL\n"	); 

			printf("\nYour choice (1 - 2) (RETURN to terminate): ");  

			gets(&buffer[0]);

			printf("\n\n");
			
			if(buffer[0] == 0)
			{
				b_terminate = true;
				
				break;
			}
			
			if( (sscanf(&buffer[0], "%d", &i_flag_choice) != 1 ) )
			{
				continue;
			}

			if( (i_flag_choice < 1)||(i_flag_choice > 2) )
			{
				continue;
			}

			break;
		}

		if(b_terminate)
		{
			break;
		}

		if(i_flag_choice == 1)
		{
			i_flag = ME_PWM_START_NO_FLAGS;

			sz_flag = "ME_PWM_START_NO_FLAGS";
		}
		else
		{
			i_flag = ME_PWM_START_CONNECT_INTERNAL;

			sz_flag = "ME_PWM_START_CONNECT_INTERNAL";
		}

		printf(	"Press RETURN to terminate or any other key to call:\n\n"
				"    meUtilityPWMStart( %-30d      (iDevice)\n"
				"                       %-30d      (iSubdevice1)\n"							
				"                       %-30d      (iSubdevice2)\n"							
				"                       %-30d      (iSubdevice3)\n"							
				"                       %-30s      (iRef)\n"							
				"                       %-30d      (iPrescaler)\n"							
				"                       %-30d      (iDutyCycle)\n"							
				"                       %-30s  );  (iFlag)\n",
				arrPWMDevices[i_list_index_device].m_iDeviceIndex,
				arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0],
				arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[1],
				arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[2],
				sz_ref,
				i_prescaler,
				i_duty_cycle,
				sz_flag													);

		getch();

		i_me_error = meUtilityPWMStart(	arrPWMDevices[i_list_index_device].m_iDeviceIndex,
							            arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0],
							            arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[1],
							            arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[2],
							            i_ref,
							            i_prescaler,
							            i_duty_cycle,
							            i_flag													);

        if(i_me_error != ME_ERRNO_SUCCESS)
        {
			printf("\n\n****    meUtilityPWMStart - Error: %d    ****\n\n", i_me_error);

			printf("Press any key to continue\n");
			
			_getch();

            continue;
	    }

		printf( "\n\nPWM Started!\n\n"
				"Press any key to call:\n\n"
				"    meUtilityPWMStop(  %-2d      (iDevice)\n"
				"                       %-2d );   (iSubdevice1)\n",							
				arrPWMDevices[i_list_index_device].m_iDeviceIndex,
				arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0]	);

		getch();

		meUtilityPWMStop(	arrPWMDevices[i_list_index_device].m_iDeviceIndex,
							arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0]	);
		
		printf( "\n\nPWM Stopped!\n\n");
	}

	delete []arrPWMDevices;

	// ---------------------------------------------------------------------
	
	meClose(0);
	
	printf("Program completed successfully - Press any key to terminate\n\n");

	_getch();
	
	return 0;
}

